# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Sandy Walsh <sandy@darksecretsoftware.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gettext
from gettext import gettext as _
gettext.textdomain('darksecret')

from gi.repository import Gtk # pylint: disable=E0611
import logging
logger = logging.getLogger('darksecret')

from darksecret_lib import Window
from darksecret.AboutDarksecretDialog import AboutDarksecretDialog
from darksecret.PreferencesDarksecretDialog import PreferencesDarksecretDialog
from darksecret import Controller

intro = """'Dark Secret Software' is a little text adventure game where you try to find the
contact information for Dark Secret Software Inc. It's a short little game, but may
grow into something larger down the road. Good luck!

"""

# See darksecret_lib.Window.py for more details about how this class works
class DarksecretWindow(Window):
    __gtype_name__ = "DarksecretWindow"

    def append(self, text):
        buff = self.history.get_buffer()
        end = buff.get_end_iter()
        buff.insert(end, text)
        self.history.scroll_mark_onscreen(buff.get_insert())

    def handle_response(self, response, context):
        exits = context['exits']
        if not response:
            self.append("%s\n" % context['description'])
            x = context['num_items']
            if x:
                if x == 1:
                    self.append("There is a %s here.\n" % context['items'])
                else:
                    self.append("There are the following items here:\n"
                                "%s\n" % context['items'])
            if exits:
                for name, room_name, room in exits:
                    name = name.capitalize()
                    self.append("To the %s you see a %s.\n" % (name, room_name))
        else:
            for raw, words in response:
                self.append("%s\n" % words)
        self.append("\n")

        directions = [name.capitalize() for name, room_name, room in exits]
        room = self.state.get_room()['short']
        exits_text = "No exits"
        if directions:
            exits_text = ', '.join(directions)
        self.status.set_text("%s Exits: %s" % (room, exits_text))

    def call_query(self, text):
        verb, target = Controller.query(self.state, text)

        response = [(False, "Sorry, I don't know that verb. Try 'help'"),]
        verbs = Controller.get_verbs()
        if verb in verbs:
            try:
                response = verbs[verb](self.state, verb=verb, target_name=target,
                                      cmds=verbs)
            except Controller.GameException, e:
                response = [(False, e.desc), ]
            except Controller.StartOver, e:
                self.state = Controller.reset()
                response = [(False, "Game reset."), ]
            except Exception, e:
                response = [(False, e), ]
        context = Controller.default_context(self.state)
        self.handle_response(response, context)

    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(DarksecretWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutDarksecretDialog
        self.PreferencesDialog = PreferencesDarksecretDialog
        self.state = Controller.State()

        # Code for other initialization actions should be added here.
        self.scroll = builder.get_object("scrolledwindow1")
        self.history = builder.get_object("history")
        self.entry = builder.get_object("entry")
        self.status = builder.get_object("status")
        self.append("Dark Secret Software v1.0 - copyright (c) Dark Secret Software Inc. 2012 ")
        for a in xrange(50):
            self.append("\n")
        self.append(intro)

        self.call_query("look")

    def on_entry_activate(self, widget):
        text = widget.get_text()
        self.append("> %s\n" % text)
        widget.set_text("")

        self.call_query(text)
